@@ -4,10 +4,12 @@
 // the terms of the BSD 3-Clause license. See the accompanying LICENSE file.
 
 #include "blas/device_blas.hh"
+#include "blas/counter.hh"
 
 #include "device_internal.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -75,6 +77,14 @@ void gemm(
         blas_error_if( ldc < n );
     }
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::dev_gemm_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { transA, transB, m, n, k };
+        counter::insert( element, counter::Id::dev_gemm );
+    #endif
+
     // convert arguments
     device_blas_int m_   = to_device_blas_int( m );
     device_blas_int n_   = to_device_blas_int( n );
