@@ -4,10 +4,12 @@
 // the terms of the BSD 3-Clause license. See the accompanying LICENSE file.
 
 #include "blas/device_blas.hh"
+#include "blas/counter.hh"
 
 #include "device_internal.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -35,6 +37,8 @@ void hemm(
 #ifndef BLAS_HAVE_DEVICE
     throw blas::Error( "device BLAS not available", __func__ );
 #else
+    using std::swap;
+
     // check arguments
     blas_error_if( layout != Layout::ColMajor &&
                    layout != Layout::RowMajor );
@@ -61,6 +65,14 @@ void hemm(
         blas_error_if( ldc < n );
     }
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::dev_hemm_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { side, uplo, m, n };
+        counter::insert( element, counter::Id::dev_hemm );
+    #endif
+
     // convert arguments
     device_blas_int m_   = to_device_blas_int( m );
     device_blas_int n_   = to_device_blas_int( n );
@@ -72,7 +84,7 @@ void hemm(
         // swap left <=> right, lower <=> upper, m <=> n
         side = (side == Side::Left  ? Side::Right : Side::Left);
         uplo = (uplo == Uplo::Lower ? Uplo::Upper : Uplo::Lower);
-        std::swap( m_, n_ );
+        swap( m_, n_ );
     }
 
     blas::internal_set_device( queue.device() );
