@@ -4,10 +4,12 @@
 // the terms of the BSD 3-Clause license. See the accompanying LICENSE file.
 
 #include "blas/device_blas.hh"
+#include "blas/counter.hh"
 
 #include "device_internal.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -56,6 +58,14 @@ void her2k(
 
     blas_error_if( ldc < n );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::dev_her2k_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, trans, n, k };
+        counter::insert( element, counter::Id::dev_her2k );
+    #endif
+
     // convert arguments
     device_blas_int n_   = to_device_blas_int( n );
     device_blas_int k_   = to_device_blas_int( k );
@@ -68,6 +78,7 @@ void her2k(
         // A => A^T; A^T => A; A^H => A
         uplo = (uplo == Uplo::Lower ? Uplo::Upper : Uplo::Lower);
         trans = (trans == Op::NoTrans ? Op::ConjTrans : Op::NoTrans);
+        alpha = conj( alpha );
     }
 
     blas::internal_set_device( queue.device() );
