@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -163,6 +165,14 @@ scalar_t dot(
     blas_error_if( incx == 0 );  // standard BLAS doesn't detect inc[xy] == 0
     blas_error_if( incy == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::dot_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { n };
+        counter::insert( element, counter::Id::dot );
+    #endif
+
     // convert arguments
     blas_int n_    = to_blas_int( n );
     blas_int incx_ = to_blas_int( incx );
@@ -189,6 +199,14 @@ scalar_t dotu(
     blas_error_if( incx == 0 );  // standard BLAS doesn't detect inc[xy] == 0
     blas_error_if( incy == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::dotu_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { n };
+        counter::insert( element, counter::Id::dotu );
+    #endif
+
     // convert arguments
     blas_int n_    = to_blas_int( n );
     blas_int incx_ = to_blas_int( incx );
