@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -107,6 +109,8 @@ void gemv(
     scalar_t beta,
     scalar_t*       y, int64_t incy )
 {
+    using std::swap;
+
     // check arguments
     blas_error_if( layout != Layout::ColMajor &&
                    layout != Layout::RowMajor );
@@ -124,6 +128,14 @@ void gemv(
     blas_error_if( incx == 0 );
     blas_error_if( incy == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::gemv_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { trans, m, n };
+        counter::insert( element, counter::Id::gemv );
+    #endif
+
     // convert arguments
     blas_int m_    = to_blas_int( m );
     blas_int n_    = to_blas_int( n );
@@ -158,10 +170,10 @@ void gemv(
             }
         }
         // A => A^T; A^T => A; A^H => A + conj
-        std::swap( m_, n_ );
+        swap( m_, n_ );
         trans2 = (trans == Op::NoTrans ? Op::Trans : Op::NoTrans);
     }
-    char trans_ = op2char( trans2 );
+    char trans_ = to_char( trans2 );
 
     // call low-level wrapper
     internal::gemv( trans_, m_, n_,
