@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -78,6 +80,8 @@ void hemm(
     scalar_t beta,
     scalar_t*       C, int64_t ldc )
 {
+    using std::swap;
+
     // check arguments
     blas_error_if( layout != Layout::ColMajor &&
                    layout != Layout::RowMajor );
@@ -104,6 +108,14 @@ void hemm(
         blas_error_if( ldc < n );
     }
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::hemm_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { side, uplo, m, n };
+        counter::insert( element, counter::Id::hemm );
+    #endif
+
     // convert arguments
     blas_int m_   = to_blas_int( m );
     blas_int n_   = to_blas_int( n );
@@ -115,10 +127,10 @@ void hemm(
         // swap left <=> right, lower <=> upper, m <=> n
         side = (side == Side::Left  ? Side::Right : Side::Left);
         uplo = (uplo == Uplo::Lower ? Uplo::Upper : Uplo::Lower);
-        std::swap( m_, n_ );
+        swap( m_, n_ );
     }
-    char side_ = side2char( side );
-    char uplo_ = uplo2char( uplo );
+    char side_ = to_char( side );
+    char uplo_ = to_char( uplo );
 
     // call low-level wrapper
     internal::hemm( side_, uplo_, m_, n_,
