@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -87,6 +89,14 @@ void hemv(
     blas_error_if( incx == 0 );
     blas_error_if( incy == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::hemv_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, n };
+        counter::insert( element, counter::Id::hemv );
+    #endif
+
     // convert arguments
     blas_int n_    = to_blas_int( n );
     blas_int lda_  = to_blas_int( lda );
@@ -118,7 +128,7 @@ void hemv(
             iy += incy;
         }
     }
-    char uplo_ = uplo2char( uplo );
+    char uplo_ = to_char( uplo );
 
     // call low-level wrapper
     internal::hemv( uplo_, n_,
