@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -74,6 +76,14 @@ void her(
     blas_error_if( lda < n );
     blas_error_if( incx == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::her_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, n };
+        counter::insert( element, counter::Id::her );
+    #endif
+
     // convert arguments
     blas_int n_    = to_blas_int( n );
     blas_int lda_  = to_blas_int( lda );
@@ -95,7 +105,7 @@ void her(
         }
         incx_ = 1;
     }
-    char uplo_ = uplo2char( uplo );
+    char uplo_ = to_char( uplo );
 
     // call low-level wrapper
     internal::her( uplo_, n_,
