@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -80,6 +82,14 @@ void her2(
     blas_error_if( incx == 0 );
     blas_error_if( incy == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::her2_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, n };
+        counter::insert( element, counter::Id::her2 );
+    #endif
+
     // convert arguments
     blas_int n_    = to_blas_int( n );
     blas_int lda_  = to_blas_int( lda );
@@ -90,7 +100,7 @@ void her2(
         // swap lower <=> upper
         uplo = (uplo == Uplo::Lower ? Uplo::Upper : Uplo::Lower);
     }
-    char uplo_ = uplo2char( uplo );
+    char uplo_ = to_char( uplo );
 
     // call low-level wrapper
     internal::her2( uplo_, n_,
