@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -99,6 +101,14 @@ void her2k(
 
     blas_error_if( ldc < n );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::her2k_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, trans, n, k };
+        counter::insert( element, counter::Id::her2k );
+    #endif
+
     // convert arguments
     blas_int n_   = to_blas_int( n );
     blas_int k_   = to_blas_int( k );
@@ -111,10 +121,11 @@ void her2k(
         // A => A^T; A^T => A; A^H => A
         uplo = (uplo == Uplo::Lower ? Uplo::Upper : Uplo::Lower);
         trans = (trans == Op::NoTrans ? Op::ConjTrans : Op::NoTrans);
+        alpha = conj( alpha );
     }
 
-    char uplo_ = uplo2char( uplo );
-    char trans_ = op2char( trans );
+    char uplo_  = to_char( uplo );
+    char trans_ = to_char( trans );
 
     // call low-level wrapper
     internal::her2k( uplo_, trans_, n_, k_,
