@@ -5,8 +5,10 @@
 
 #include "blas/fortran.h"
 #include "blas.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -21,6 +23,14 @@ void rotg(
     float *c,
     float *s )
 {
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::rotg_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { 1 };
+        counter::insert( element, counter::Id::rotg );
+    #endif
+
     BLAS_srotg( a, b, c, s );
 }
 
@@ -32,6 +42,14 @@ void rotg(
     double *c,
     double *s )
 {
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::rotg_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { 1 };
+        counter::insert( element, counter::Id::rotg );
+    #endif
+
     BLAS_drotg( a, b, c, s );
 }
 
@@ -43,6 +61,14 @@ void rotg(
     float *c,
     std::complex<float> *s )
 {
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::rotg_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { 1 };
+        counter::insert( element, counter::Id::rotg );
+    #endif
+
     BLAS_crotg( (blas_complex_float*) a,
                 (blas_complex_float*) b,
                 c,
@@ -57,6 +83,14 @@ void rotg(
     double *c,
     std::complex<double> *s )
 {
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::rotg_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { 1 };
+        counter::insert( element, counter::Id::rotg );
+    #endif
+
     BLAS_zrotg( (blas_complex_double*) a,
                 (blas_complex_double*) b,
                 c,
