@@ -5,8 +5,10 @@
 
 #include "blas/fortran.h"
 #include "blas.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -23,6 +25,14 @@ void rotmg(
     float  b,
     float  param[5] )
 {
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::rotmg_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { 1 };
+        counter::insert( element, counter::Id::rotmg );
+    #endif
+
     BLAS_srotmg( d1, d2, a, &b, param );
 }
 
@@ -35,6 +45,14 @@ void rotmg(
     double  b,
     double  param[5] )
 {
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::rotmg_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { 1 };
+        counter::insert( element, counter::Id::rotmg );
+    #endif
+
     BLAS_drotmg( d1, d2, a, &b, param );
 }
 
