@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -73,6 +75,14 @@ void syr2(
     blas_error_if( incx == 0 );
     blas_error_if( incy == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::syr2_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, n };
+        counter::insert( element, counter::Id::syr2 );
+    #endif
+
     // convert arguments
     blas_int n_    = to_blas_int( n );
     blas_int lda_  = to_blas_int( lda );
@@ -86,7 +96,7 @@ void syr2(
 
     if constexpr (! is_complex<scalar_t>::value) {
         // call low-level wrapper
-        char uplo_ = uplo2char( uplo );
+        char uplo_ = to_char( uplo );
         internal::syr2( uplo_, n_, alpha, x, incx_, y, incy_, A, lda_ );
     }
     else { // is_complex<scalar_t>
@@ -99,15 +109,15 @@ void syr2(
         // elif inc >= 1, consider x and y as 1-by-n matrices in inc-by-n arrays,
         // else, copy x and y and use case inc == 1 above.
         blas_int k_ = 1;
-        char trans_;
+        Op trans_;
         blas_int ldx_, ldy_;
         if (incx == 1 && incy == 1) {
-            trans_ = 'N';
+            trans_ = Op::NoTrans;
             ldx_ = n_;
             ldy_ = n_;
         }
         else if (incx >= 1 && incy >= 1) {
-            trans_ = 'T';
+            trans_ = Op::Trans;
             ldx_ = incx_;
             ldy_ = incy_;
         }
@@ -122,14 +132,14 @@ void syr2(
                 ix += incx;
                 iy += incy;
             }
-            trans_ = 'N';
+            trans_ = Op::NoTrans;
             ldx_ = n_;
             ldy_ = n_;
         }
         scalar_t beta = 1;
 
         // call low-level wrapper
-        syr2k( blas::Layout::ColMajor, uplo, char2op(trans_), n_, k_,
+        syr2k( blas::Layout::ColMajor, uplo, trans_, n_, k_,
                          alpha, x2, ldx_, y2, ldy_, beta, A, lda_ );
 
         if (x2 != x) {
