@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -141,6 +143,14 @@ void syr2k(
 
     blas_error_if( ldc < n );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::syr2k_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, trans, n, k };
+        counter::insert( element, counter::Id::syr2k );
+    #endif
+
     // convert arguments
     blas_int n_   = to_blas_int( n );
     blas_int k_   = to_blas_int( k );
@@ -154,8 +164,8 @@ void syr2k(
         uplo = (uplo == Uplo::Lower ? Uplo::Upper : Uplo::Lower);
         trans = (trans == Op::NoTrans ? Op::Trans : Op::NoTrans);
     }
-    char uplo_ = uplo2char( uplo );
-    char trans_ = op2char( trans );
+    char uplo_  = to_char( uplo );
+    char trans_ = to_char( trans );
 
     // call low-level wrapper
     internal::syr2k( uplo_, trans_, n_, k_,
