@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -108,6 +110,8 @@ void trmm(
     scalar_t const* A, int64_t lda,
     scalar_t*       B, int64_t ldb )
 {
+    using std::swap;
+
     // check arguments
     blas_error_if( layout != Layout::ColMajor &&
                    layout != Layout::RowMajor );
@@ -133,6 +137,14 @@ void trmm(
     else
         blas_error_if( ldb < n );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::trmm_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { side, uplo, trans, diag, m, n };
+        counter::insert( element, counter::Id::trmm );
+    #endif
+
     // convert arguments
     blas_int m_   = to_blas_int( m );
     blas_int n_   = to_blas_int( n );
@@ -143,12 +155,12 @@ void trmm(
         // swap lower <=> upper, left <=> right, m <=> n
         uplo = (uplo == Uplo::Lower ? Uplo::Upper : Uplo::Lower);
         side = (side == Side::Left ? Side::Right : Side::Left);
-        std::swap( m_, n_ );
+        swap( m_, n_ );
     }
-    char side_  = side2char( side );
-    char uplo_  = uplo2char( uplo );
-    char trans_ = op2char( trans );
-    char diag_  = diag2char( diag );
+    char side_  = to_char( side );
+    char uplo_  = to_char( uplo );
+    char trans_ = to_char( trans );
+    char diag_  = to_char( diag );
 
     // call low-level wrapper
     internal::trmm( side_, uplo_, trans_, diag_, m_, n_,
