@@ -6,8 +6,10 @@
 #include "blas/fortran.h"
 #include "blas.hh"
 #include "blas_internal.hh"
+#include "blas/counter.hh"
 
 #include <limits>
+#include <string.h>
 
 namespace blas {
 
@@ -108,6 +110,14 @@ void trsv(
     blas_error_if( lda < n );
     blas_error_if( incx == 0 );
 
+    #ifdef BLAS_HAVE_PAPI
+        // PAPI instrumentation
+        counter::trsv_type element;
+        memset( &element, 0, sizeof( element ) );
+        element = { uplo, trans, diag, n };
+        counter::insert( element, counter::Id::trsv );
+    #endif
+
     // convert arguments
     blas_int n_    = to_blas_int( n );
     blas_int lda_  = to_blas_int( lda );
@@ -131,9 +141,9 @@ void trsv(
             }
         }
     }
-    char uplo_  = uplo2char( uplo );
-    char trans_ = op2char( trans2 );
-    char diag_  = diag2char( diag );
+    char uplo_  = to_char( uplo );
+    char trans_ = to_char( trans2 );
+    char diag_  = to_char( diag );
 
     // call low-level wrapper
     internal::trsv( uplo_, trans_, diag_, n_, A, lda_, x, incx_ );
