@@ -9,23 +9,55 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "blas/counter.hh"
+
+#ifdef BLAS_HAVE_PAPI
+    #include "papi.h"
+#endif
+
 #include "test.hh"
 
+// Headers for versions.
+#ifdef BLAS_HAVE_MKL
+    #include <mkl.h>
+#endif
+#ifdef BLAS_HAVE_OPENBLAS
+    #include <cblas.h>
+#endif
+#ifdef BLAS_HAVE_CUBLAS
+    #include <cuda_runtime.h>
+#endif
+#ifdef BLAS_HAVE_ROCBLAS
+    #include <rocm-core/rocm_version.h>
+#endif
+
 // -----------------------------------------------------------------------------
 using testsweeper::ParamType;
 using testsweeper::DataType;
-using testsweeper::char2datatype;
-using testsweeper::datatype2char;
-using testsweeper::datatype2str;
+using testsweeper::DataType_help;
+
 using testsweeper::ansi_bold;
 using testsweeper::ansi_red;
 using testsweeper::ansi_normal;
 
-const double no_data = testsweeper::no_data_flag;
+using blas::Layout, blas::Layout_help;
+using blas::Side,   blas::Side_help;
+using blas::Uplo,   blas::Uplo_help;
+using blas::Op,     blas::Op_help;
+using blas::Diag,   blas::Diag_help;
+using blas::Format;
 
-// const ParamType PT_Value  = ParamType::Value; currently unused
-// const ParamType PT_List   = ParamType::List; currently unused
-const ParamType PT_Output = ParamType::Output;
+const char* Format_help = "one of: L or LAPACK; T or Tile";
+
+const ParamType PT_Value = ParamType::Value;
+const ParamType PT_List  = ParamType::List;
+const ParamType PT_Out   = ParamType::Output;
+
+const double no_data = testsweeper::no_data_flag;
+const char*  pi_rt2i = "3.141592653589793 + 1.414213562373095i";
+const char*  e_rt3i  = "2.718281828459045 + 1.732050807568877i";
+const double pi      = 3.141592653589793;
+const double e       = 2.718281828459045;
 
 // -----------------------------------------------------------------------------
 // each section must have a corresponding entry in section_names
@@ -199,73 +231,77 @@ Params::Params():
 
     // w = width
     // p = precision
-    // def = default
-    // ----- test framework parameters
-    //         name,       w,    type,         default, valid, help
-    check     ( "check",   0,    ParamType::Value, 'y', "ny",  "check the results" ),
-    ref       ( "ref",     0,    ParamType::Value, 'n', "ny",  "run reference; sometimes check -> ref" ),
+    //----- test framework parameters
+    //          name,         w, type, default, valid, help
+    check     ( "check",      0, PT_Value, 'y', "ny", "check the results" ),
+    ref       ( "ref",        0, PT_Value, 'n', "ny", "run reference; sometimes check implies ref" ),
+    papi      ( "papi",       0, PT_Value, 'n', "ny", "run papi instrumentation" ),
 
-    //          name,      w, p, type,         default, min,  max, help
-    repeat    ( "repeat",  0,    ParamType::Value,   1,   1, 1000, "times to repeat each test" ),
-    verbose   ( "verbose", 0,    ParamType::Value,   0,   0,   10, "verbose level" ),
-    cache     ( "cache",   0,    ParamType::Value,  20,   1, 1024, "total cache size, in MiB" ),
+    //          name,         w, p, type, default,  min,  max, help
+    repeat    ( "repeat",     0,    PT_Value,   1,    1, 1000, "times to repeat each test" ),
+    verbose   ( "verbose",    0,    PT_Value,   0,    0,   10, "verbose level" ),
+    cache     ( "cache",      0,    PT_Value,  20,    1, 1024, "total cache size, in MiB" ),
 
-    // ----- routine parameters
-    //          name,      w,    type,            def,                    char2enum,         enum2char,         enum2str,         help
-    datatype  ( "type",    4,    ParamType::List, DataType::Double,       char2datatype,     datatype2char,     datatype2str,     "s=single (float), d=double, c=complex-single, z=complex-double" ),
-    layout    ( "layout",  6,    ParamType::List, blas::Layout::ColMajor, blas::char2layout, blas::layout2char, blas::layout2str, "layout: r=row major, c=column major" ),
-    format    ( "format",  6,    ParamType::List, blas::Format::LAPACK,   blas::char2format, blas::format2char, blas::format2str, "format: l=lapack, t=tile" ),
-    side      ( "side",    6,    ParamType::List, blas::Side::Left,       blas::char2side,   blas::side2char,   blas::side2str,   "side: l=left, r=right" ),
-    uplo      ( "uplo",    6,    ParamType::List, blas::Uplo::Lower,      blas::char2uplo,   blas::uplo2char,   blas::uplo2str,   "triangle: l=lower, u=upper" ),
-    trans     ( "trans",   7,    ParamType::List, blas::Op::NoTrans,      blas::char2op,     blas::op2char,     blas::op2str,     "transpose: n=no-trans, t=trans, c=conj-trans" ),
-    transA    ( "transA",  7,    ParamType::List, blas::Op::NoTrans,      blas::char2op,     blas::op2char,     blas::op2str,     "transpose of A: n=no-trans, t=trans, c=conj-trans" ),
-    transB    ( "transB",  7,    ParamType::List, blas::Op::NoTrans,      blas::char2op,     blas::op2char,     blas::op2str,     "transpose of B: n=no-trans, t=trans, c=conj-trans" ),
-    diag      ( "diag",    7,    ParamType::List, blas::Diag::NonUnit,    blas::char2diag,   blas::diag2char,   blas::diag2str,   "diagonal: n=non-unit, u=unit" ),
+    //----- routine parameters, enums
+    //          name,         w, type,    default, help
+    datatype  ( "type",       4, PT_List, DataType::Double, DataType_help ),
 
-    //          name,      w, p, type,            def,   min,     max, help
-    dim       ( "dim",     6,    ParamType::List,          0,     1e9, "m by n by k dimensions" ),
-    alpha     ( "alpha",   9, 4, ParamType::List,  pi,  -inf,     inf, "scalar alpha" ),
-    beta      ( "beta",    9, 4, ParamType::List,   e,  -inf,     inf, "scalar beta" ),
-    incx      ( "incx",    4,    ParamType::List,   1, -1000,    1000, "stride of x vector" ),
-    incy      ( "incy",    4,    ParamType::List,   1, -1000,    1000, "stride of y vector" ),
-    align     ( "align",   0,    ParamType::List,   1,     1,    1024, "column alignment (sets lda, ldb, etc. to multiple of align)" ),
-    batch     ( "batch",   6,    ParamType::List, 100,     0,     1e6, "batch size" ),
-    device    ( "device",  6,    ParamType::List,   0,     0,     100, "device id" ),
-    pointer_mode ( "pointer-mode",  3,    ParamType::List, 'h',  "hd",          "h == host, d == device" ),
+    // BLAS & LAPACK options
+    layout    ( "layout",     6, PT_List, Layout::ColMajor, Layout_help ),
+    format    ( "format",     6, PT_List, Format::LAPACK, Format_help ),
+    side      ( "side",       6, PT_List, Side::Left, Side_help ),
+    uplo      ( "uplo",       6, PT_List, Uplo::Lower, Uplo_help ),
+    trans     ( "trans",      7, PT_List, Op::NoTrans, Op_help ),
+    transA    ( "transA",     7, PT_List, Op::NoTrans, Op_help ),
+    transB    ( "transB",     7, PT_List, Op::NoTrans, Op_help ),
+    diag      ( "diag",       7, PT_List, Diag::NonUnit, Diag_help ),
+    pointer_mode( "ptr",      3, PT_List, 'h', "hd", "one of: h or host; d or device" ),
 
-    // ----- output parameters
+    //----- routine parameters, numeric
+    //          name,         w, p, type,    default,  min,  max, help
+    dim       ( "dim",        6,    PT_List,             0, 1e10, "m by n by k dimensions" ),
+    alpha     ( "alpha",      3, 1, PT_List, pi_rt2i, -inf,  inf, "scalar alpha" ),
+    beta      ( "beta",       3, 1, PT_List,  e_rt3i, -inf,  inf, "scalar beta" ),
+    incx      ( "incx",       4,    PT_List,       1, -1e3,  1e3, "stride of x vector" ),
+    incy      ( "incy",       4,    PT_List,       1, -1e3,  1e3, "stride of y vector" ),
+    align     ( "align",      0,    PT_List,       1,    1, 1024, "column alignment (sets lda, ldb, etc. to multiple of align)" ),
+    batch     ( "batch",      6,    PT_List,     100,    0,  1e6, "batch size" ),
+    device    ( "device",     6,    PT_List,       0,    0,  100, "device id" ),
+
+    //----- output parameters
     // min, max are ignored
-    //          name,            w, p, type,      default, min, max, help
-    // error: %8.2e allows 9.99e-99
-    error     ( "error",         8, 2, PT_Output, no_data, 0, 0, "numerical error" ),
-    error2    ( "error2",        8, 2, PT_Output, no_data, 0, 0, "numerical error 2" ),
-    error3    ( "error3",        8, 2, PT_Output, no_data, 0, 0, "numerical error 3" ),
-
-    // time:    %9.3f allows 99999.999 s = 2.9 days (ref headers need %12)
+    // error:   %8.2e allows 9.99e-99
+    // time:    %9.3f allows 99999.999 s = 2.9 days
     // gflops: %12.3f allows 99999999.999 Gflop/s = 100 Pflop/s
-    time      ( "time (s)",      9, 3, PT_Output, no_data, 0, 0, "time to solution" ),
-    gflops    ( "gflop/s",      12, 3, PT_Output, no_data, 0, 0, "Gflop/s rate" ),
-    gbytes    ( "gbyte/s",      12, 3, PT_Output, no_data, 0, 0, "Gbyte/s rate" ),
+    //          name,         w, p, type,   default, min, max, help
+    error     ( "error",      8, 2, PT_Out, no_data, 0, 0, "numerical error" ),
+    error2    ( "error2",     8, 2, PT_Out, no_data, 0, 0, "numerical error" ),
+    error3    ( "error3",     8, 2, PT_Out, no_data, 0, 0, "numerical error" ),
 
-    time2     ( "time (s)",      9, 3, PT_Output, no_data, 0, 0, "time to solution (2)" ),
-    gflops2   ( "gflop/s",      12, 3, PT_Output, no_data, 0, 0, "Gflop/s rate (2)" ),
-    gbytes2   ( "gbyte/s",      12, 3, PT_Output, no_data, 0, 0, "Gbyte/s rate (2)" ),
+    time      ( "time (s)",   9, 3, PT_Out, no_data, 0, 0, "time to solution" ),
+    gflops    ( "gflop/s",   12, 3, PT_Out, no_data, 0, 0, "Gflop/s rate" ),
+    gbytes    ( "gbyte/s",   12, 3, PT_Out, no_data, 0, 0, "Gbyte/s rate" ),
 
-    time3     ( "time (s)",      9, 3, PT_Output, no_data, 0, 0, "time to solution (3)" ),
-    gflops3   ( "gflop/s",      12, 3, PT_Output, no_data, 0, 0, "Gflop/s rate (3)" ),
-    gbytes3   ( "gbyte/s",      12, 3, PT_Output, no_data, 0, 0, "Gbyte/s rate (3)" ),
+    time2     ( "time (s)",   9, 3, PT_Out, no_data, 0, 0, "extra timer" ),
+    gflops2   ( "gflop/s",   12, 3, PT_Out, no_data, 0, 0, "Gflop/s rate" ),
+    gbytes2   ( "gbyte/s",   12, 3, PT_Out, no_data, 0, 0, "Gbyte/s rate" ),
 
-    time4     ( "time (s)",      9, 3, PT_Output, no_data, 0, 0, "time to solution (4)" ),
-    gflops4   ( "gflop/s",      12, 3, PT_Output, no_data, 0, 0, "Gflop/s rate (4)" ),
-    gbytes4   ( "gbyte/s",      12, 3, PT_Output, no_data, 0, 0, "Gbyte/s rate (4)" ),
+    time3     ( "time (s)",   9, 3, PT_Out, no_data, 0, 0, "extra timer" ),
+    gflops3   ( "gflop/s",   12, 3, PT_Out, no_data, 0, 0, "Gflop/s rate" ),
+    gbytes3   ( "gbyte/s",   12, 3, PT_Out, no_data, 0, 0, "Gbyte/s rate" ),
 
-    ref_time  ( "ref time (s)", 12, 3, PT_Output, no_data, 0, 0, "reference time to solution" ),
-    ref_gflops( "ref gflop/s",  12, 3, PT_Output, no_data, 0, 0, "reference Gflop/s rate" ),
-    ref_gbytes( "ref gbyte/s",  12, 3, PT_Output, no_data, 0, 0, "reference Gbyte/s rate" ),
+    time4     ( "time (s)",   9, 3, PT_Out, no_data, 0, 0, "extra timer" ),
+    gflops4   ( "gflop/s",   12, 3, PT_Out, no_data, 0, 0, "Gflop/s rate" ),
+    gbytes4   ( "gbyte/s",   12, 3, PT_Out, no_data, 0, 0, "Gbyte/s rate" ),
+
+    ref_time  ( "ref time (s)",  9, 3, PT_Out, no_data, 0, 0, "reference time to solution" ),
+    ref_gflops( "ref gflop/s",  12, 3, PT_Out, no_data, 0, 0, "reference Gflop/s rate" ),
+    ref_gbytes( "ref gbyte/s",  12, 3, PT_Out, no_data, 0, 0, "reference Gbyte/s rate" ),
 
     // default -1 means "no check"
-    okay      ( "status",              6,    ParamType::Output,  -1,   0,   0, "success indicator" ),
-    msg       ( "",       1, ParamType::Output,  "",           "error message" )
+    //          name,         w, type, default, min, max, help
+    okay      ( "status",     6, PT_Out,    -1, 0, 0, "success indicator" ),
+    msg       ( "",           1, PT_Out,    "",       "error message" )
 {
     // set header different than command line prefix
     pointer_mode.name("ptr", "pointer-mode");
@@ -281,25 +317,83 @@ Params::Params():
     repeat();
     verbose();
     cache();
+    papi();
 
     // routine's parameters are marked by the test routine; see main
 }
 
+// -----------------------------------------------------------------------------
+void setup_PAPI( int *event_set )
+{
+    blas::counter::get();
+    #ifdef BLAS_HAVE_PAPI
+        require( PAPI_library_init( PAPI_VER_CURRENT ) == PAPI_VER_CURRENT );
+
+        require( PAPI_create_eventset( event_set ) == PAPI_OK );
+
+        require( PAPI_add_named_event( *event_set, "sde:::blas::counter" ) == PAPI_OK );
+    #endif
+}
+
 // -----------------------------------------------------------------------------
 int main( int argc, char** argv )
 {
     using testsweeper::QuitException;
 
+    // These may or may not be used; mark unused to silence warnings.
+    blas_unused( pi_rt2i );
+    blas_unused( e_rt3i  );
+    blas_unused( pi      );
+    blas_unused( e       );
+
     // check that all sections have names
     require( sizeof(section_names)/sizeof(*section_names) == Section::num_sections );
 
     int status = 0;
     try {
         int version = blas::blaspp_version();
-        printf( "BLAS++ version %d.%02d.%02d, id %s\n",
+        printf( "BLAS++ version %d.%02d.%02d, id %s",
                 version / 10000, (version % 10000) / 100, version % 100,
                 blas::blaspp_id() );
 
+        // CPU BLAS
+        // todo: Accelerate, IBM ESSL, Cray LibSci, BLIS, etc. version.
+        #ifdef BLAS_HAVE_ACCELERATE
+            printf( ", Apple Accelerate" );
+        #endif
+        #ifdef BLAS_HAVE_ESSL
+            printf( ", IBM ESSL" );
+        #endif
+        #ifdef BLAS_HAVE_MKL
+            MKLVersion mkl_version;
+            mkl_get_version( &mkl_version );
+            printf( ", Intel MKL %d.%d.%d",
+                    mkl_version.MajorVersion,
+                    mkl_version.MinorVersion,
+                    mkl_version.UpdateVersion );
+        #endif
+        #ifdef BLAS_HAVE_OPENBLAS
+            printf( ", %s", OPENBLAS_VERSION );
+        #endif
+
+        // GPU BLAS
+        #ifdef BLAS_HAVE_CUBLAS
+            printf( ", CUDA %d.%d.%d",
+                    CUDART_VERSION / 1000,
+                    (CUDART_VERSION % 1000) / 100,
+                    CUDART_VERSION % 10 );
+        #endif
+        #ifdef BLAS_HAVE_ROCBLAS
+            printf( ", ROCm %d.%d.%d",
+                    ROCM_VERSION_MAJOR,
+                    ROCM_VERSION_MINOR,
+                    ROCM_VERSION_PATCH );
+        #endif
+        #ifdef BLAS_HAVE_SYCL
+            printf( ", SYCL" );
+        #endif
+        printf( "\n" );
+
         // print input so running `test [input] > out.txt` documents input
         printf( "input: %s", argv[0] );
         for (int i = 1; i < argc; ++i) {
@@ -350,6 +444,18 @@ int main( int argc, char** argv )
             params.align.width( 5 );
         }
 
+        #ifdef BLAS_HAVE_PAPI
+            int event_set = PAPI_NULL;
+            long long counter_values[1];
+
+            if (params.papi() == 'y') {
+                // initialize papi
+                setup_PAPI( &event_set );
+
+                require( PAPI_start( event_set ) == PAPI_OK );
+            }
+        #endif
+
         // run tests
         int repeat = params.repeat();
         testsweeper::DataType last = params.datatype();
@@ -379,6 +485,17 @@ int main( int argc, char** argv )
             }
         } while(params.next());
 
+        #ifdef BLAS_HAVE_PAPI
+            if (params.papi() == 'y') {
+                // stop papi
+                require( PAPI_stop( event_set, counter_values ) == PAPI_OK );
+
+                // print papi instrumentation
+                blas::counter::print( (cset_list_object_t *)counter_values[0] );
+                printf( "\n" );
+            }
+        #endif
+
         if (status) {
             printf( "%d tests FAILED for %s.\n", status, routine );
         }
