@@ -15,13 +15,14 @@ template <typename TX, typename TY>
 void test_axpy_work( Params& params, bool run )
 {
     using namespace testsweeper;
+    using std::abs;
     using std::real;
     using std::imag;
     using scalar_t = blas::scalar_type< TX, TY >;
     using real_t   = blas::real_type< scalar_t >;
 
     // get & mark input values
-    scalar_t alpha  = params.alpha();
+    scalar_t alpha  = params.alpha.get<scalar_t>();
     int64_t n       = params.dim.n();
     int64_t incx    = params.incx();
     int64_t incy    = params.incy();
@@ -113,8 +114,8 @@ void test_axpy_work( Params& params, bool run )
         int64_t ix = (incx > 0 ? 0 : (-n + 1)*incx);
         int64_t iy = (incy > 0 ? 0 : (-n + 1)*incy);
         for (int64_t i = 0; i < n; ++i) {
-            y[iy] = std::abs( y[iy] - yref[iy] )
-                  / (2*(std::abs( alpha * x[ix] ) + std::abs( y0[iy] )));
+            y[iy] = abs( y[iy] - yref[iy] )
+                  / (2*(abs( alpha * x[ix] ) + abs( y0[iy] )));
             error = std::max( error, real( y[iy] ) );
             ix += incx;
             iy += incy;
