@@ -20,6 +20,7 @@ void test_batch_trsm_work( Params& params, bool run )
     using blas::Layout;
     using scalar_t = blas::scalar_type< TA, TB >;
     using real_t   = blas::real_type< scalar_t >;
+    using std::swap;
 
     // get & mark input values
     blas::Layout layout = params.layout();
@@ -27,7 +28,7 @@ void test_batch_trsm_work( Params& params, bool run )
     blas::Uplo uplo_    = params.uplo();
     blas::Op trans_    = params.trans();
     blas::Diag diag_    = params.diag();
-    scalar_t alpha_     = params.alpha();
+    scalar_t alpha_     = params.alpha.get<scalar_t>();
     int64_t m_          = params.dim.m();
     int64_t n_          = params.dim.n();
     size_t  batch       = params.batch();
@@ -47,7 +48,7 @@ void test_batch_trsm_work( Params& params, bool run )
     int64_t Bm = m_;
     int64_t Bn = n_;
     if (layout == Layout::RowMajor)
-        std::swap( Bm, Bn );
+        swap( Bm, Bn );
     int64_t lda_ = roundup( Am, align );
     int64_t ldb_ = roundup( Bm, align );
     size_t size_A = size_t(lda_)*Am;
@@ -109,7 +110,7 @@ void test_batch_trsm_work( Params& params, bool run )
             Aarray[s][ i + i*lda_ ] += Am;
         }
         int64_t blas_info = 0;
-        lapack_potrf( uplo2str(uplo_), Am, Aarray[s], lda_, &blas_info );
+        lapack_potrf( to_c_string( uplo_ ), Am, Aarray[s], lda_, &blas_info );
         require( blas_info == 0 );
     }
 
@@ -119,7 +120,7 @@ void test_batch_trsm_work( Params& params, bool run )
     real_t* Bnorm = new real_t[ batch ];
 
     for (size_t s = 0; s < batch; ++s) {
-        Anorm[s] = lapack_lantr( "f", uplo2str(uplo_), diag2str(diag_), Am, Am, Aarray[s], lda_, work );
+        Anorm[s] = lapack_lantr( "f", to_c_string( uplo_ ), to_c_string( diag_ ), Am, Am, Aarray[s], lda_, work );
         Bnorm[s] = lapack_lange( "f", Bm, Bn, Barray[s], ldb_, work );
     }
 
@@ -128,7 +129,7 @@ void test_batch_trsm_work( Params& params, bool run )
         for (size_t s = 0; s < batch; ++s) {
             for (int64_t j = 0; j < Am; ++j) {
                 for (int64_t i = 0; i < j; ++i) {
-                    std::swap( Aarray[s][ i + j*lda_ ], Aarray[s][ j + i*lda_ ] );
+                    swap( Aarray[s][ i + j*lda_ ], Aarray[s][ j + i*lda_ ] );
                 }
             }
         }
