@@ -27,8 +27,8 @@ void test_hemm_work( Params& params, bool run )
     blas::Layout layout = params.layout();
     blas::Side side = params.side();
     blas::Uplo uplo = params.uplo();
-    scalar_t alpha  = params.alpha();
-    scalar_t beta   = params.beta();
+    scalar_t alpha  = params.alpha.get<scalar_t>();
+    scalar_t beta   = params.beta.get<scalar_t>();
     int64_t m       = params.dim.m();
     int64_t n       = params.dim.n();
     int64_t align   = params.align();
@@ -68,7 +68,7 @@ void test_hemm_work( Params& params, bool run )
 
     // norms for error check
     real_t work[1];
-    real_t Anorm = lapack_lansy( "f", uplo2str(uplo), An, A, lda, work );
+    real_t Anorm = lapack_lansy( "f", to_c_string( uplo ), An, A, lda, work );
     real_t Bnorm = lapack_lange( "f", Cm, Cn, B, ldb, work );
     real_t Cnorm = lapack_lange( "f", Cm, Cn, C, ldc, work );
 
@@ -94,7 +94,7 @@ void test_hemm_work( Params& params, bool run )
                 "A An=%5lld, An=%5lld, lda=%5lld, size=%10lld, norm %.2e\n"
                 "B  m=%5lld,  n=%5lld, ldb=%5lld, size=%10lld, norm %.2e\n"
                 "C  m=%5lld,  n=%5lld, ldc=%5lld, size=%10lld, norm %.2e\n",
-                side2char(side), uplo2char(uplo),
+                to_char( side ), to_char( uplo ),
                 llong( An ), llong( An ), llong( lda ), llong( size_A ), Anorm,
                 llong( m ), llong( n ), llong( ldb ), llong( size_B ), Bnorm,
                 llong( m ), llong( n ), llong( ldc ), llong( size_C ), Cnorm );
