@@ -22,6 +22,7 @@ void test_trsm_work( Params& params, bool run )
     using blas::Diag;
     using scalar_t = blas::scalar_type< TA, TB >;
     using real_t   = blas::real_type< scalar_t >;
+    using std::swap;
 
     // get & mark input values
     blas::Layout layout = params.layout();
@@ -29,7 +30,7 @@ void test_trsm_work( Params& params, bool run )
     blas::Uplo uplo = params.uplo();
     blas::Op trans  = params.trans();
     blas::Diag diag = params.diag();
-    scalar_t alpha  = params.alpha();
+    scalar_t alpha  = params.alpha.get<scalar_t>();
     int64_t m       = params.dim.m();
     int64_t n       = params.dim.n();
     int64_t align   = params.align();
@@ -49,7 +50,7 @@ void test_trsm_work( Params& params, bool run )
     int64_t Bm = m;
     int64_t Bn = n;
     if (layout == Layout::RowMajor)
-        std::swap( Bm, Bn );
+        swap( Bm, Bn );
     int64_t lda = roundup( Am, align );
     int64_t ldb = roundup( Bm, align );
     size_t size_A = size_t(lda)*Am;
@@ -83,12 +84,12 @@ void test_trsm_work( Params& params, bool run )
         A[ i + i*lda ] += Am;
     }
     int64_t info = 0;
-    lapack_potrf( uplo2str(uplo), Am, A, lda, &info );
+    lapack_potrf( to_c_string( uplo ), Am, A, lda, &info );
     require( info == 0 );
 
     // norms for error check
     real_t work[1];
-    real_t Anorm = lapack_lantr( "f", uplo2str(uplo), diag2str(diag),
+    real_t Anorm = lapack_lantr( "f", to_c_string( uplo ), to_c_string( diag ),
                                  Am, Am, A, lda, work );
     real_t Bnorm = lapack_lange( "f", Bm, Bn, B, ldb, work );
 
@@ -96,7 +97,7 @@ void test_trsm_work( Params& params, bool run )
     if (layout == Layout::RowMajor) {
         for (int64_t j = 0; j < Am; ++j) {
             for (int64_t i = 0; i < j; ++i) {
-                std::swap( A[ i + j*lda ], A[ j + i*lda ] );
+                swap( A[ i + j*lda ], A[ j + i*lda ] );
             }
         }
     }
